--
-- File            	: top.vhd
-- Author          : Detzi
-- Date            : 12/10/2016
-- Version         : 0.00
-- Abstract        : top level top
--
-- Modification History:
-- Date        By       Version    Change Description
--
-- ===========================================================
-- 00/00/00    xxx      x.x        Original 
-- 
-- ===========================================================
-- Library and Use statements for IEEE packages
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all; 
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
 
entity top is  

port( 
	TasterA 	: in	std_logic;					-- Taster A
	TasterB 	: in  	std_logic;					-- Taster B
	D1			: out std_logic;					-- LED 1
	D2			: out std_logic;					-- LED 2
	D3			: out std_logic;					-- LED 3
	D4			: out std_logic;					-- LED 4
	D5			: out std_logic;					-- LED 5
	D6			: out std_logic;					-- LED 6
	D7			: out std_logic;					-- LED 7
	D8			: out std_logic;					-- LED 8
	SCL			: out std_logic;					-- DAC Transfer clock
	SOut		: out std_logic;					-- data Stream Output
	Sync		: out std_logic;					-- Sync Signal for DAC
	TP1			: out std_logic;
	TP2			: out std_logic
	);
	
end top;

architecture top_arch of top is
	--Oszillator
	signal osc_clk 		: std_logic;														--Main system clock
	signal sysReset 	: std_logic;														--Main reset
	signal RegEmpty 	: std_logic;														--Buffer empty and needs more data
	signal DACData		: std_logic_vector(23 downto 0);									--Dac voltage data Point
	signal compclk		: std_logic;														--dac output freq*24	
	signal start		: std_logic;														-- Binds the key to the start command
	signal sck_clk		: std_logic;														--dac output freq*24	
	

	component OSCH										-- Internal Osc
        generic (
            NOM_FREQ : string);
        port (
            STDBY         	: in  std_logic := 'X';
            OSC, SEDSTDBY 	: out std_logic := 'X');
    end component;	
	
	component SiCoSiTable is
    port (
        Clock: in  std_logic; 
        ClkEn: in  std_logic; 
        Reset: in  std_logic; 
        Theta: in  std_logic_vector(7 downto 0); 
        Sine: out  std_logic_vector(15 downto 0); 
        Cosine: out  std_logic_vector(15 downto 0));
	end component SiCoSiTable;
	
	
	component Prescaler is    
	generic (
		divider : natural);	
	port(	
	compclk: in std_logic;								--Clock with witch the Data are Output
	clkout: out std_logic); 								--Enables the Clock Input
	end component;
	
	component PrescalerSCK is    
	generic (
		divider : natural);	
	port(	
	compclk: in std_logic;								--Clock with witch the Data are Output
	clkout: out std_logic); 								--Enables the Clock Input
	end component;
	
	component Trigger is    
	generic (
			cycles : natural 
			);	
	port(	
		compclk: in std_logic;
		key: in std_logic;									--Clock with witch the Data are Output
		cmd: buffer std_logic								--Enables the Clock Input
	); 
	end component;
	
	component PISOsr is
		generic (
				size : natural
				);	
		port(	
			compclk: in std_logic;								--Clock with witch the Data are Output
			assign: in std_logic:='1';								--Assigns the data on 'dout' to the Register deletes old data
			empty: out std_logic;							--Signals that the 24'th bit ist visible on 'dout'
			din: in std_logic_vector(size downto 0);	--Data Input
			dout: out std_logic									--Data Output
		); 
	end component PISOsr;
	
	
begin
	DACData(23 downto 16)<=x"11"; 													-- 00 Normal operation, 01 Power-down 1 k to GND, 10 Power-down 100 k? to GND, 11 Power-down High-Z
	D1<=start;
	D2<= '1';
	D3<= '1';
	D4<= '1';
	D5<= '1';
	D6<= '1';
	D7<= not TasterA;
	D8<= not TasterB; 
	sysReset<='0';
	Sync<=RegEmpty;
	SCL<=sck_clk;
	TP1<=compclk;
	TP2<=osc_clk;
	OSC_inst : OSCH 
	generic map(NOM_FREQ => "2.08 ") 
	port map(	
		STDBY => '0', 
		OSC => osc_clk, 
		SEDSTDBY => open
	);
	
	i_SiCoSiTable:	SiCoSiTable
    port map (
        Clock => '1', 
        ClkEn=> '0',
        Reset=> sysReset,
        Theta=> x"00",
        Sine=> open,--DACData (15 downto 0),
        Cosine=> open
	);
	
	i_PISOsr:	PISOsr
		generic map (	size => 23)	
		port map(	
			compclk => compclk, 
			assign=> start, 
			empty=> RegEmpty, 
			din => DACData, 							--Data Input
			dout=> SOut 									--Data Output
		); 
		
	i_Prescaler: Prescaler     
	generic map(divider =>1000)	
	port map(	
	compclk=> osc_clk, 
	clkout=>compclk								--Enables the Clock Input
	);
	
	i_PrescalerSCK: PrescalerSCK     
	generic map(divider =>10)	
	port map(	
	compclk=> compclk, 
	clkout=>sck_clk								--Enables the Clock Input
	);
	
	i_Trigger: Trigger 
	generic map (cycles =>100)
	port map(	
		compclk=>compclk,	
		key=>TasterA,									--Clock with witch the Data are Output
		cmd=>start									--Enables the Clock Input
	); 
	
end top_arch;